/*
	StringControl.cpp
	© 1997 Vikram Kulkarni (vkulkarn@uiuc.edu)
	11/6/97 - Wrote it
	11/7/97 - Added comments

	To see what a string control does
	look at the TestStringControl app
	that should have come with this.
                                        */


#include "StringControl.h"

StringControl::StringControl	(// passed to the BView constructor
								 BRect frame,
								 // passed to the BView constructor
								 char * name,
								 // the text that's shown
								 char * label,
								 // the message that's sent
								 BMessage * message,
								 // the text color when the mouse is over the control
								 // OR when the control is the Focus view
								 // A blueish color by default
								 const rgb_color * glow_color,
								 // the text color at other times
								 // black by default
								 const rgb_color * text_color,
								 // the font
								 // be_plain_font by default
								 const BFont * font,
								 // passed to the BView constructor
								 uint32 resizingMode,
								 // passed to the BView constructor
								 uint32 flags)
	: BControl(frame, name, label, message, resizingMode, flags)
{
	fenabled = true;
	fglowing = false;
	ffocused = false;
	
	if (!font)
		// Using default font
		ffont = new BFont( be_plain_font );
	else
		// Using passed font
		ffont = new BFont( font );
	
	// Setting font
	// Use BView::SetFont to change
	// Use BView::GetFont to get
	SetFont(ffont);
	
	if (!text_color)
	{
		// Using default color (black)
		ftext_color = new rgb_color;
		ftext_color->red = 0;
		ftext_color->green = 0;
		ftext_color->blue = 0;
	}
	else
	{
		// Using passed color
		ftext_color = new rgb_color;
		memcpy(ftext_color, text_color, sizeof(rgb_color));
	}
	
	// Setting text color
	SetHighColor(*ftext_color);
	
	if (!glow_color)
	{
		// Using default color
		fglow_color = new rgb_color;
		fglow_color->red = 0;
		fglow_color->green = 100;
		fglow_color->blue = 255;
	}
	else
	{
		// Using passed color
		fglow_color = new rgb_color;
		memcpy(fglow_color, glow_color, sizeof(rgb_color));
	}

	// Setting draw_point
	// !!!WARNING!!! ther is no check to see if the top of
	// !!!WARNING!!! the text is cut off.
	// !!!WARNING!!! USE ResizeToPreferred() to make sure
	// !!!WARNING!!! that the tops aren't cut off, or make
	// !!!WARNING!!! sure that your frame is the correct size
	font_height temp_height;
	ffont->GetHeight(&temp_height);

	draw_point.Set(1, frame.Height());
	draw_point.y -= (temp_height.descent + 1.0);
}

StringControl::~StringControl()
{
	// Cleaning house
	if (ffont)
		delete (ffont);
	if (ftext_color)
		delete (ftext_color);
	if (fglow_color)
		delete (fglow_color);
}

void StringControl::Draw(BRect updateRect)
{
	BView::Draw(updateRect);
	
	// Drawing the label
	DrawString(Label(), draw_point);
}

void StringControl::MakeFocus( bool focused )
{
	BView::MakeFocus( focused );
	if (focused == ffocused)
		return;
	else
	{
		// sets ffocused
		ffocused = focused;
		if (ffocused)
			// if its the Focus View it will glow
			// even without the mouse being over it
			SetHighColor(*fglow_color);
		else
			// if it's not the focus view it shouldn't
			// glow anymore
			SetHighColor(*ftext_color);
		Invalidate();
	}
}

void StringControl::MouseDown(BPoint where)
{
	// sends the message
	BControl::Invoke();
}

void StringControl::MouseMoved(BPoint where,
								uint32 transit,
								const BMessage *message)
{
	// The control can't deal with any dragged messages so it
	//  wont glow if a message is dragged over it.
	// There's no need to keep reseting the high color so
	//  I ignore the B_INSIDE_VIEW.
	// And of course, it must be enabled to glow
	if ((message == NULL) && (transit != B_INSIDE_VIEW) && fenabled)
	{
		// if the mouse is entering the view GLOW
		if (transit == B_ENTERED_VIEW)
		{
			fglowing = true;
			SetHighColor(*fglow_color);
			Invalidate();
		}
		// if it's exiting the view and it's not the focus view
		// stop glowing
		else if ((transit == B_EXITED_VIEW) && !ffocused)
		{
			fglowing = false;
			SetHighColor(*ftext_color);
			Invalidate();
		}
		
	}
}

void StringControl::GetPreferredSize(float * width, float * height)
{
	// The width is easy... BFont does it.
	// I ad a 2 unit spacer.
	*width = ffont->StringWidth(Label()) + 2.0;
	// Get the font height
	font_height temp_height;
	ffont->GetHeight(&temp_height);
	// how tall is the font + a 2 unit spacer.
	*height = temp_height.ascent + temp_height.descent + 2.0;
	
}

void StringControl::ResizeToPreferred()
{
	float width, height;
	
	// get the Width
	width = ffont->StringWidth(Label()) + 2.0;

	// get the height
	font_height temp_height;
	ffont->GetHeight(&temp_height);
	height = temp_height.ascent + temp_height.descent + 2.0;
	
	// and Resize to it
	ResizeTo(width, height);

	// and reset draw_point so the text is placed nicely
	draw_point.Set(1, temp_height.ascent + 1.0);

	Invalidate();
}

void StringControl::SetEnabled( bool enabled)
{
	BControl::SetEnabled(enabled);

	// Reset color
	if (!fenabled && enabled)
		SetHighColor(*ftext_color);
		
	// Set fenabled
	fenabled = enabled;
		
	Invalidate();
}

void StringControl::SetTextColor( rgb_color new_text_color )
{
	// delete current ftext_color
	if (ftext_color)
		delete (ftext_color);
	
	// copy current new_text_color	
	ftext_color = new rgb_color;
	memcpy(ftext_color, &new_text_color, sizeof(rgb_color));
	
	// if it's not glowing reset the high color
	if (!fglowing)
	{
		SetHighColor(*ftext_color);
		Invalidate();
	}
}

void StringControl::SetTextColor( uint8 red, uint8 green, uint8 blue, uint8 alpha )
{
	// set the new text color
	ftext_color->red = red;
	ftext_color->green = green;
	ftext_color->blue = blue;
	ftext_color->alpha = alpha;
	
	// if it's not glowing reset the high color
	if (!fglowing)
	{
		SetHighColor(*ftext_color);
		Invalidate();
	}
}

rgb_color StringControl::TextColor() const
{
	// return the text color
	return (*ftext_color);
}

void StringControl::SetGlowColor( rgb_color new_glow_color )
{
	// delete the current glow color
	if (fglow_color)
		delete (fglow_color);

	// copy new text color
	fglow_color = new rgb_color;
	memcpy(fglow_color, &new_glow_color, sizeof(rgb_color));
	
	// if it's glowing reset high color
	if (fglowing)
	{
		SetHighColor(*fglow_color);
		Invalidate();
	}
}

void StringControl::SetGlowColor( uint8 red, uint8 green, uint8 blue, uint8 alpha )
{
	// set new glow color
	fglow_color->red = red;
	fglow_color->green = green;
	fglow_color->blue = blue;
	fglow_color->alpha = alpha;

	// if it's glowing reset high color
	if (fglowing)
	{
		SetHighColor(*fglow_color);
		Invalidate();
	}
}

rgb_color StringControl::GlowColor() const
{
	// return the glow color
	return (*fglow_color);
}
